// === Model classes and keys ===
var PushHandler = require('cloud/PushHandler.js');
var RankingHandler = require('cloud/RankingHandler.js');
var UserHandler = require('cloud/UserHandler.js');


var _matchClassName = "Match";
var _matchLockKey = "gameLock";
var _matchLockKeyInitial = 1;
var _matchLockKeyMax = 2;
var _matchPlayer1Key = "player1";
var _matchPlayer2Key = "player2";
var _matchStatusKey = "gameStatus";
var _matchStatusKeyWaiting = "waiting";
var _matchStatusKeyInProgress = "in_progress";
var _matchStatusKeyFinished = "finished";
var _matchStatusKeyCancelled = "cancelled";
var _matchTurnKey = "turn";
var _matchTurnKeyPlayer1 = "player_1";
var _matchTurnKeyPlayer2 = "player_2";
var _isCurrent = "isCurrent";
var _isCurrentInitial = "true";
var _matchCategory = "matchCategory";
var _matchData = "matchData";
var _towerDataKeyPlayer1 = "towerDataPlayer1";
var _towerDataKeyPlayer2 = "towerDataPlayer2";
var _matchWinLoss = "matchWinLoss";
var _matchIsWaiting = "isWaiting";
var _player1Score = "player1Score";
var _player2Score = "player2Score";
var _installationClassName = Parse.Installation;
var Match = Parse.Object.extend(_matchClassName);
// === Overridable methods ===
// TODO all these
exports.initialize = function() {
};
exports.beforeMatchCreated = function() {
};
exports.afterMatchCreated = function() {
};
exports.beforeMatchJoined = function() {
};
exports.afterMatchJoined = function() {
};
exports.beforeTurnChange = function() {
};
exports.afterTurnChange = function() {
};
// === API methods ===




exports.pushToOther = function(request, options) {
    var matchQuery = new Parse.Query(Match);
    matchQuery.include(_matchPlayer1Key);
    matchQuery.include(_matchPlayer2Key);
    matchQuery.get(request.params.matchId, {
        success: function(matchFound) {
            // object is an instance of Parse.Object.

            var defaultPlayerKey = "player1";
            var secondPlayerKey = "player2";
            var defaultPlayer = matchFound.get(defaultPlayerKey);
            if (defaultPlayer.id != request.user.id) {
                defaultPlayerKey = "player2";
                secondPlayerKey = "player1";
                defaultPlayer = matchFound.get(defaultPlayerKey);
            }
            var otherPlayer = matchFound.get(secondPlayerKey);
            var matchValuesForPush = {"condition": "Panding Game", "matchId": matchFound.id};
            PushHandler._sendPush(otherPlayer, otherPlayer.get("name") + " Invite You to complete a pending Game.", matchValuesForPush);
            options.success(true);
        },
        error: options.error});
};

_getMatches = function(request, options, tMateches, player1) {
    var player2 = request.user;
    var match = Parse.Object.extend(_matchClassName);
    var lotsOfWins = new Parse.Query(match);
    lotsOfWins.include(_matchPlayer1Key);
    lotsOfWins.include(_matchPlayer2Key);
    lotsOfWins.equalTo("player1", request.user);
    var OfWins = new Parse.Query(match);
    OfWins.include(_matchPlayer1Key);
    OfWins.include(_matchPlayer2Key);
    OfWins.equalTo("player2", request.user);
    var mainQuery = Parse.Query.or(lotsOfWins, OfWins);
    mainQuery.include(_matchPlayer1Key);
    mainQuery.include(_matchPlayer2Key);
    mainQuery.limit(1000);
    mainQuery.select("player1Score", "player2Score", "isWaiting", "gameStatus", "isCurrent", "player1", "player2", "matchCategory");
    mainQuery.find({
        success: function(results) {
            // results is an array of Parse.Object.
            console.log("results.." + results.length);
            console.log(results);
            if (tMateches) {
                if (tMateches > results.length) {
                    //send push to creator that your req. is rejected.
                    var matchValuesForPush = {"condition": "MatchRejected", "matchId": "match deleted"};
                    PushHandler._sendPush(player1, player2.get("name") + " rejected your challenge", matchValuesForPush);
                    options.success(results);
                } else {
                    _getMatches(request, options, tMateches, player1);
                }
            } else {
                options.success(results);
            }


        },
        error: options.error});
};
exports.getMyMatches = function(request, options) {

    _getMatches(request, options);
};


exports.joinAnonymousGame = function(obj, options) {
    var player = obj.user;
    var categoryName = "Questions"; //obj.params.categoryName;
    console.log("categoryName got :" + categoryName);
    _log("Joining anonymous game", player);
    // Find random player

    var match = Parse.Object.extend("Match");
    if (!match) {
        _createNewRandomMatch(player, null, categoryName, options);
    } else {
        var query = new Parse.Query(match);
        query.include(_matchPlayer1Key);
        query.include(_matchPlayer2Key);
        query.notEqualTo(_matchPlayer1Key, player);
        query.equalTo(_matchPlayer2Key, null);
        query.equalTo(_matchIsWaiting, true);
//        query.include(_matchData);
        query.find({
            success: function(objects) {
                console.log(objects.length);
                if (objects.length > 0) {
                    var player2 = player;
                    var match = objects[0];
                    var player1 = match.get(_matchPlayer1Key);
                    console.log(player1.id + ' - ' + player1.get('username'));
                    _joinRandomMatch(match, player2, player1, options);
                } else {
                    _createNewRandomMatch(player, null, categoryName, options);
                }
            },
            error: function(error) {
                console.log("An error occured :( in random match finding.");
                options.error(error);
            }
        });
    }//end of else of if (!match)
};
_joinRandomMatch = function(match, player2, player1, options) {

    match.set(_matchPlayer2Key, player2);
    match.set(_matchIsWaiting, false);
    match.save(null, {
        success: function(newMatch) {
            // Return the game
            var matchValuesForPush = {"condition": "MatchJoined", "matchId": newMatch.id};
            PushHandler._sendPush(player1, player2.get("name") + " join you", matchValuesForPush);
            _reTurnMatchWithFormat(newMatch, options);

        },
        error: options.error
    });
};
//if not any change in it or _joinRandomMatch then will be use only one and it will be remove
_joinChallengeMatch = function(match, player2, player1, options) {


    match.set(_matchPlayer2Key, player2);
    match.set(_matchIsWaiting, false);
    match.save(null, {
        success: function(newMatch) {
            // Return the game
            var matchValuesForPush = {"condition": "ChallengeJoined", "matchId": newMatch.id};
            PushHandler._sendPush(player1, player2.get("name") + " accepted your challenge", matchValuesForPush);
            _reTurnMatchWithFormat(newMatch, options);
//            options.success(newMatch);
        },
        error: options.error
    });
};


exports.joinMatch = function(request, options)
{

    var player2 = request.user;
    var matchId = request.params.matchId;
    var match = Parse.Object.extend(_matchClassName);
    var query = new Parse.Query(match);
    query.include(_matchPlayer1Key);
    query.include(_matchPlayer2Key);
//    query.include(_matchData);

    query.get(matchId, {
        success: function(myMatch)
        {
            var player1 = myMatch.get(_matchPlayer1Key);
            _joinChallengeMatch(myMatch, player2, player1, options);
        },
        error: options.error
    });
};




_matchAfterSave = function(request, options) {
    // code here

    Parse.Cloud.useMasterKey();
    var player = request.user;
    var matchId = request.params.matchId;
//    var matchScore = request.params.matchScore;
    var match = Parse.Object.extend(_matchClassName);
    var query = new Parse.Query(match);
    query.include(_matchPlayer1Key);
    query.include(_matchPlayer2Key);
    query.get(request.params.matchId, {
        success: function(myMatch) {

            var defaultPlayerKey = "player1";
            var secondPlayerKey = "player2";
            var defaultPlayer = myMatch.get(defaultPlayerKey);
            var secondPlayer = myMatch.get(secondPlayerKey);
            var player1GameStatus = defaultPlayer.get(_matchStatusKey);
            var player2GameStatus = secondPlayer.get(_matchStatusKey);
            if (!player1GameStatus) {
                player1GameStatus = {};
                player1GameStatus["wins"] = 0;
                player1GameStatus["draws"] = 0;
                player1GameStatus["losses"] = 0;
            }
            if (!player2GameStatus) {
                player2GameStatus = {};
                player2GameStatus["wins"] = 0;
                player2GameStatus["draws"] = 0;
                player2GameStatus["losses"] = 0;
            }

            var pushReceiver;
            var pushSender;
            if (defaultPlayer.id != player.id) {
                //myMatch.set(_player2Score,matchScore);
                pushReceiver = defaultPlayer;
                pushSender = secondPlayer;
            } else {
                //myMatch.set(_player1Score,matchScore);
                pushReceiver = secondPlayer;
                pushSender = defaultPlayer;
            }

            console.log("defaultPlayer" + defaultPlayer);
            console.log(defaultPlayer);
            console.log("secondPlayer" + secondPlayer);
            console.log(secondPlayer);
            var score1 = myMatch.get(_player1Score);
            var score2 = myMatch.get(_player2Score);
            var player1Score;
            var player2Score;
            if (score1 && score2) {
                player1Score = parseInt(score1, 10);
                player2Score = parseInt(score2, 10);
            }

            var isLastPlayerScore = false;
            var resultMessage = "";
            if (typeof player1Score == "number" && typeof player2Score == "number") {
                isLastPlayerScore = true;
                myMatch.set(_isCurrent, "false");
                if (player1Score == player2Score) {
                    myMatch.set(_matchStatusKey, "tie");
                    resultMessage = "Quiz tie between you and " + pushSender.get("name");
                    player1GameStatus["draws"] = player1GameStatus["draws"] + 1;
                    player2GameStatus["draws"] = player2GameStatus["draws"] + 1;
                    defaultPlayer.set(_matchStatusKey, player1GameStatus);
                    secondPlayer.set(_matchStatusKey, player2GameStatus);
                } else if (player1Score > player2Score) {
                    myMatch.set(_matchStatusKey, "player1")
                    if (player.id == defaultPlayer.id) {//push will be send to player2
                        resultMessage = "You lose from " + defaultPlayer.get("name");
                    } else {//push will be send to player1
                        resultMessage = "You win from " + secondPlayer.get("name");
                    }
                    player1GameStatus["wins"] = player1GameStatus["wins"] + 1;
                    player2GameStatus["losses"] = player2GameStatus["losses"] + 1;
                    defaultPlayer.set(_matchStatusKey, player1GameStatus);
                    secondPlayer.set(_matchStatusKey, player2GameStatus);
                } else {
                    myMatch.set(_matchStatusKey, "player2");
                    if (player.id == defaultPlayer.id) {//push will be send to player2
                        resultMessage = "You win from " + defaultPlayer.get("name");
                    } else {//push will be send to player1
                        resultMessage = "You lose from " + secondPlayer.get("name");
                    }
                    player1GameStatus["losses"] = player1GameStatus["losses"] + 1;
                    player2GameStatus["wins"] = player2GameStatus["wins"] + 1;
                    defaultPlayer.set(_matchStatusKey, player1GameStatus);
                    secondPlayer.set(_matchStatusKey, player2GameStatus);
                }

                var player1CumulativeScore = defaultPlayer.get("cumulativeScore");
                var player2CumulativeScore = secondPlayer.get("cumulativeScore");
                defaultPlayer.set("cumulativeScore", player1CumulativeScore + player1Score);
                if (secondPlayer.get("isAgent") != true) {
                    secondPlayer.set("cumulativeScore", player2CumulativeScore + player2Score);
                }


                var optionsForCountryRank = {
                    success: function(res) {
                        console.log("optionsForCountryRank");
                        console.log(res);
                    },
                    error: function(error) {
                        console.log("error optionsForCountryRank");
                        console.log(error);
                    }
                };

                RankingHandler._updateCountryRank(defaultPlayer.get("countryName"), player1Score, optionsForCountryRank);
                if (secondPlayer.get("isAgent") != true) {
                    RankingHandler._updateCountryRank(secondPlayer.get("countryName"), player2Score, optionsForCountryRank);
                }

            }


            myMatch.save(null, {
                success: function(newMatch) {
                    // The save was successful.
                    console.log("match score set successfully...");
                    defaultPlayer.save(null, {
                        success: function(playerOne) {
                            _log("playerOne GameStatus updated ", playerOne);
                            secondPlayer.save(null, {
                                success: function(playerTwo) {
                                    _log("playerTwo GameStatus updated ", playerTwo);
                                    if (isLastPlayerScore == true) {
                                        var matchValuesForPush = {"condition": "MatchCompleted", "matchId": newMatch.id};
                                        PushHandler._sendPush(pushReceiver, resultMessage, matchValuesForPush);
                                    } else {
                                        var matchValuesForPush = {"condition": "MatchScore", "matchId": newMatch.id};
                                        PushHandler._sendPush(pushReceiver, player.get("name") + " submitted his score. Now it's your turn", matchValuesForPush);
                                    }
                                    options.success(newMatch);
                                },
                                error: function(error)
                                {
                                    // The object was not retrieved successfully.
                                    // error is a Parse.Error with an error code and description.
                                    console.log(" error in playerTwo match setScore  ...");
                                    console.log(error);
                                    options.error(error);
                                }
                            });
                        },
                        error: function(error)
                        {
                            // The object was not retrieved successfully.
                            // error is a Parse.Error with an error code and description.
                            console.log(" error in playerOne match setScore  ...");
                            console.log(error);
                            options.error(error);
                        }
                    });
                },
                error: function(error)
                {
                    // The object was not retrieved successfully.
                    // error is a Parse.Error with an error code and description.
                    console.log(" error in matchsave match setScore  ...");
                    console.log(error);
                    options.error(error);
                }
            });
        },
        error: function(error) {
            console.error("Got an error in afterSave" + error.code + " : " + error.message);
        }
    });
};


exports.setMyScore = function(request, options) {

    Parse.Cloud.useMasterKey();
    var player = request.user;
    var matchId = request.params.matchId;
    var matchScore = request.params.matchScore;
    var match = Parse.Object.extend(_matchClassName);
    var query = new Parse.Query(match);
    query.include(_matchPlayer1Key);
    query.include(_matchPlayer2Key);
    query.get(matchId, {
        success: function(myMatch)
        {
            // The object was retrieved successfully.
            var defaultPlayerKey = "player1";
            var secondPlayerKey = "player2";
            var defaultPlayer = myMatch.get(defaultPlayerKey);
            var secondPlayer = myMatch.get(secondPlayerKey);
            var pushReceiver;
            if (defaultPlayer.id != player.id) {
                myMatch.set(_player2Score, matchScore);
                pushReceiver = defaultPlayer;
            } else {
                myMatch.set(_player1Score, matchScore);
                pushReceiver = secondPlayer;
            }


            myMatch.save(null, {
                success: function(newMatch) {
                    _matchAfterSave(request, options);
                    console.log("match score set successfully...");
                },
                error: function(error)
                {
                    // The object was not retrieved successfully.
                    // error is a Parse.Error with an error code and description.
                    console.log(" error in matchsave match setScore  ...");
                    console.log(error);
                    options.error(error);
                }
            });
        },
        error: function(error)
        {
            // The object was not retrieved successfully.
            // error is a Parse.Error with an error code and description.
            console.log(" error in get match  setScore  ...");
            console.log(error);
            options.error(error);
        }
    });
};


_getQuestionsById = function(ids, options) {
    console.log("ids");
    console.log(ids);
    randomCategory = "Questions";
    var randCategoryObject = Parse.Object.extend(randomCategory);
    var query = new Parse.Query(randCategoryObject);
    query.containedIn("objectId", ids);
    query.find({
        success: function(results) {
            console.log("results");
            console.log(results);
            var newArrayOfQuestions = [];
            for (key in results) {
                newArrayOfQuestions[key] = results[key];
            }
            options.success(newArrayOfQuestions);
        },
        error: function(error) {
            options.error(error + " error in _getQuestionsById");
        }
    });
};


_getQuestionsByIndex = function(ids, options) {//ids now there are indexes in it
    console.log("ids");
    console.log(ids);
    randomCategory = "Questions";
    var randCategoryObject = Parse.Object.extend(randomCategory);
    var query = new Parse.Query(randCategoryObject);
    query.containedIn("Index", ids);//ids now there are indexes in it
    query.find({
        success: function(results) {
            console.log("results");
            console.log(results);
            var newArrayOfQuestions = [];
            for (key in results) {
                newArrayOfQuestions[key] = results[key];
            }
            options.success(newArrayOfQuestions);
        },
        error: function(error) {
            options.error(error + " error in _getQuestionsById");
        }
    });
};

exports.getFiveQuestions = function(request, options) {
    var player = request.user;
    var matchData = {};
    var randomCategory = "";
    --request.params.category;
//    if (!randomCategory) {
    randomCategory = "Questions";
//    }

    var query = new Parse.Query(randomCategory);
    query.count({
        success: function(number) {
            var arr = [];
            var randomLimits = {};
            randomLimits.pUpper = number;
            var lower = 1;
            if (player) {
                randomLimits = null;
                randomLimits = player.get("randomLimits");
                lower = randomLimits.pUpper - 4;
            }
            var randomLimit = 5;
            while (arr.length < randomLimit) {
                var randomnumber = Math.floor(Math.random() * (randomLimits.pUpper - lower + 1)) + lower;

                var found = false;
                for (var i = 0; i < arr.length; i++) {
                    if (arr[i] == randomnumber) {
                        found = true;
                        break
                    }
                }
                if (!found)
                    arr[arr.length] = randomnumber;
            }

            var queryPickvalue = new Parse.Query(randomCategory);
            console.log("randomLimits.lower ,randomLimits.upper in queryPickvalue in getfive questions " + randomLimits.lower + " " + randomLimits.upper);
            queryPickvalue.containedIn("Index", arr);
            queryPickvalue.find({
                success: function(results) {
                    // results is an array of Parse.Object.
                    console.log("results in queryPickvalue  in getfive questions  " + results);
                    console.log("results.length in queryPickvalue  in getfive questions  " + results.length);
                    for (var j = 0; j < arr.length; j++) {
                        console.log("Arr  " + arr[j]);
                        matchData[j] = results[j]; //results[arr[j] - 1];
                        console.log(" matchData[j] in getfive questions   " + matchData[j]);
                    }

                    randomLimits.pUpper = randomLimits.pUpper + 5;
                    if (randomLimits.pUpper > number) {
                        randomLimits.pUpper = 5;
                    }
                    var keyValues = [];
                    keyValues["randomLimits"] = randomLimits;
                    if (player) {
                        UserHandler.updateUser(keyValues, player, {
                            success: function(isUpadeted) {
                                options.success(matchData);
                            },
                            error: function(error) {
                                options.error(error);
                            }
                        });
                    } else {
                        options.success(matchData);
                    }
                },
                error: function(error) {
                    console.log("queryPickvalue.find( main error hai ");
                    options.error(error);
                }
            });
        },
        error: function(error) {
            console.log("query.count main error hai  ");
            options.error(error);
        }
    }
    );
};

_reTurnMatchWithFormat = function(newMatch, options) {

    var matchData = newMatch.get(_matchData);
    var player1 = newMatch.get("player1");
    var player2 = newMatch.get("player2");
    var newMyMatch = newMatch;
    var newOptions = {
        success: function(result) {
            var matchInJson = newMyMatch.toJSON();
            matchInJson.matchData = result;
            matchInJson.player1 = player1;
            matchInJson.player2 = player2;
            options.success(matchInJson);
        },
        error: function(error) {
            options.error("not find questions.");
        }
    };
//    var matchInJson=newMatch.toJSON();
//    console.log("matchInJson" + matchInJson.objectId);
//    matchInJson.objectId=null;
//     console.log("matchInJson" + matchInJson.objectId);
//    console.log( matchInJson);
//    console.log("matchData" + matchData.length);
//    console.log(matchData);
    var indexes = [];//ids now there are indexes in it
    for (var key in matchData) {
        indexes[key] = matchData[key];
    }

    if (matchData[0].id != null) {
        var ids = [];
        for (var key in matchData) {
            ids[key] = matchData[key].id;
        }
        _getQuestionsById(ids, newOptions);
    } else {
        _getQuestionsByIndex(indexes, newOptions);
    }

};


exports.getMatch = function(request, options) {
    var user = request.params.user;
    var matchId = request.params.matchId;
    var mainQuery = new Parse.Query(Match);
    mainQuery.include(_matchPlayer1Key);
    mainQuery.include(_matchPlayer2Key);
//    mainQuery.include(_matchData);
    mainQuery.equalTo("objectId", matchId);
    mainQuery.find({
        success: function(results) {
            // results is an array of Parse.Object.
            console.log("getMatch success");
            console.log("results.." + results.length);
            console.log(results[0]);
            if (results.length > 0 && results[0] != null) {
                _reTurnMatchWithFormat(results[0], options);
            } else {
                options.error("Quiz have been removed.");
            }
//            options.success(results);
        },
        error: options.error});
};


_createNewRandomMatch = function(player, player2, category, options) {
    // Set new match attributes

    var match = new (Parse.Object.extend(_matchClassName))();
    var matchData = {};
    var randomCategory = category;
//    if (!randomCategory) {
    randomCategory = "Questions";
//    }

    var query = new Parse.Query(randomCategory);
    query.count({
        success: function(number) {
            var randomLimits = player.get("randomLimits");
            //randomLimits.upper = 5;//will be remove on final code
            //randomLimits.lower = 1;//will be remove on final code
            var arr = [];
            var randomLimit = 5;
            while (arr.length < randomLimit) {
                var randomnumber = Math.floor(Math.random() * (randomLimits.upper - randomLimits.lower + 1)) + randomLimits.lower;
                var found = false;
                for (var i = 0; i < arr.length; i++) {
                    if (arr[i] == randomnumber) {
                        found = true;
                        break
                    }
                }
                if (!found)
                    arr[arr.length] = randomnumber;
            }

            var queryPickvalue = new Parse.Query(randomCategory);
            console.log("randomLimits.lower ,randomLimits.upper in queryPickvalue  " + randomLimits.lower + " " + randomLimits.upper);
//            queryPickvalue.limit(randomLimits.upper);
//            queryPickvalue.skip(randomLimits.lower-1);
            queryPickvalue.containedIn("Index", arr);
            queryPickvalue.find({
                success: function(results) {
                    // results is an array of Parse.Object.
                    console.log("results in queryPickvalue  " + results);
                    console.log("results.length in queryPickvalue  " + results.length);
                    for (var j = 0; j < arr.length; j++) {
                        console.log("Arr  " + arr[j]);
                        var index = results[j].get("Index");
                        if (index == null || index <= 0) {
                            index = Math.floor(Math.random() * (10 - 1 + 1)) + 1;//get an index from 1st 10 questions
                        }
                        matchData[j] = index; //results[arr[j] - 1];
                        console.log(" matchData[j]  " + matchData[j]);
                    }
                    if (player2) {
                        match.set(_matchPlayer2Key, player2);
                    }
                    match.set(_matchPlayer1Key, player);
                    match.set(_matchLockKey, _matchLockKeyInitial);
                    match.set(_matchIsWaiting, true); // wait for second player
                    match.set(_matchTurnKey, _matchTurnKeyPlayer1); // default challenger starts
                    match.set(_isCurrent, _isCurrentInitial);
                    match.set(_matchCategory, randomCategory);
                    match.set(_matchData, matchData);
                    _log("Creating new game with properties:", player);
//                    _log(JSON.stringify(match), player);
                    // Create match
                    match.save({
                        success: function(newMatch) {
                            // Return the game
                            _log("Game created successfully", player);
                            _log(JSON.stringify(newMatch), player);
                            if (player2) {
                                var matchValuesForPush = {"condition": "ChallengeCreated", "matchId": newMatch.id};
                                PushHandler._sendPush(player2, player.get("name") + " challenge you to play the Cricket Quiz", matchValuesForPush);
                            }
//                            will be umcoment on final code
                            randomLimits.lower = randomLimits.upper + 1;
                            randomLimits.upper = randomLimits.upper + 10;
                            if (randomLimits.upper > number) {
                                randomLimits.upper = 10;
                                randomLimits.lower = 1;
                            }
                            var keyValues = [];
                            keyValues["randomLimits"] = randomLimits;
                            UserHandler.updateUser(keyValues, player, {
                                success: function(isUpadeted) {
                                    _reTurnMatchWithFormat(newMatch, options);
//                                    options.success(newMatch);
                                },
                                error: function(error) {
                                    options.error("updateUser error in _createNewRandomMatch");
                                }
                            });
                        },
                        error: function(error) {
                            console.log("save match error in _createNewRandomMatch");
                            options.error(error);
                        }
                    });
                },
                error: function(error) {
                    // error is an instance of Parse.Error.
                    console.log(" Parse.Error inner");
                }
            });
        },
        error: function(error) {
            // error is an instance of Parse.Error.
            console.log(" Parse.Error outer");
        }
    });
};



_isChallengePandingSendPush = function(player1, player2, options) {

    Parse.Cloud.useMasterKey();
    var matchQuery = new Parse.Query(Match);
//    matchQuery.include(_matchPlayer1Key);
//    matchQuery.include(_matchPlayer2Key);
    var player1Ptr = {"__type": "Pointer", "className": "_User", "objectId": player1.id};
    var player2Ptr = {"__type": "Pointer", "className": "_User", "objectId": player2.id};
    matchQuery.equalTo(_matchPlayer1Key, player1Ptr);
    matchQuery.equalTo(_matchPlayer2Key, player2Ptr);
    matchQuery.equalTo(_matchIsWaiting, true);
    matchQuery.limit(1);
    matchQuery.first({
        success: function(matchFound) {
            // object is an instance of Parse.Object.
            console.log("matchFound");
            console.log(matchFound);
            if (matchFound == null) {
                options.success(false);
            } else {
                var matchValuesForPush = {"condition": "Panding Game", "matchId": matchFound.id};
                PushHandler._sendPush(player2, player1.get("name") + " wants to remind you to accept the challenge.", matchValuesForPush);
                options.success(true);
            }
        },
        error: options.error});

};


exports.rematch = function(player1, player2id, categoryName, options) {

    var player2 = null;

    var callbackOptions = {
        success: function(yesOrNo) {
            if (yesOrNo == false) {
                _createNewRandomMatch(player1, player2, categoryName, options);
            } else if (yesOrNo == true) {
                console.log("push is send to other user for panding challenge.");
                options.success("reminderSend");
            }
        },
        error: function(error) {
            options.error(error + ": player2 not find.");
        }
    };

    var getUserCallback = {
        success: function(playerTwo) {
            if (playerTwo.get("isAgent") != true) {
                player2 = playerTwo;
                _isChallengePandingSendPush(player1, player2, callbackOptions);
            } else {
                var error=[];
                error["code"]="cannotChallenge";
                error["error"]="You could not challenge to " + playerTwo.get("name") + ", please try later.";
                options.error(error);
            }
        },
        error: function(error) {
            options.error(error + ": player2 not find.");
        }
    };
    var query = new Parse.Query("User");
    query.get(player2id, getUserCallback);


};

_log = function(message, player) {
    console.log(player.get("username") + "$ " + message);
};


_getRandomAgent = function(options) {
    var query = new Parse.Query("User");
    query.equalTo("isAgent", true);
    query.find({
        success: function(results) {
            console.log("results in _getRandomAgent" + results.length);
            console.log(results);
            for (var k in results) {
                console.log("k:" + k);
            }
            var length = results.length - 1;
            var random = Math.floor(Math.random() * (length - 0 + 1)) + 0;
            var agent = results[random];
            console.log("agent in _getRandomAgent");
            console.log(agent);
            options.success(agent);
        },
        error: function(error) {
            console.log("error in _getRandomAgent");
            options.error(error);
        }
    });
};

_matchWithAgent = function(match, options) {
    Parse.Cloud.useMasterKey();
    var callBackOptions = {
        success: function(agent) {
            console.log("agent in callBackOptions");
            console.log(agent);
            var randomScore = Math.floor(Math.random() * (40 - 10 + 1)) + 10;
            console.log("randomScore in callBackOptions");
            console.log(randomScore);
            match.set(_player2Score, "" + randomScore + "");
            match.set(_matchPlayer2Key, agent);
            match.set(_matchIsWaiting, false);
            match.save(null, {
                success: function(newMatch) {
                    // Return the game
                    _reTurnMatchWithFormat(newMatch, options);
//            options.success(newMatch);
                },
                error: function(err) {
                    console.log("error in match.save");
                    console.log(err);
                    options.error(err);
                }
            });

        },
        error: function(err) {
            options.error(err);
        }
    };
    _getRandomAgent(callBackOptions);
};



exports.matchOrDelete = function(matchId, isCancel, options)
{

    var match = Parse.Object.extend(_matchClassName);
    var query = new Parse.Query(match);
    query.include(_matchPlayer1Key);
    query.include(_matchPlayer2Key);
    query.get(matchId, {
        success: function(myObj)
        {

            if (myObj.get(_matchIsWaiting) == false) {
                console.log("match has been joined successfully...");
                _reTurnMatchWithFormat(myObj, options);
//                options.success(myObj);
            } else {
                if (isCancel == true) {
                    myObj.destroy({});
                    console.log("match deleted successfully...");
                    options.success(false);
                } else {
                    _matchWithAgent(myObj, options);
                }
            }
        },
        error: function(object, error)
        {
            // The object was not retrieved successfully.
            // error is a Parse.Error with an error code and description.
            console.log("error in match deleting ...");
            console.log(error);
            options.error(true);
        }
    });
};





exports.deleteMatch = function(request, options)
{
    var matchId = request.params.matchId;
    var match = Parse.Object.extend(_matchClassName);
    var query = new Parse.Query(match);
// query.equalTo("objectId",matchId)


    query.get(matchId, {
        success: function(myObj)
        {
            // The object was retrieved successfully.
            var player1 = myObj.get("player1");
            myObj.destroy({});
            console.log("match deleted successfully...");
//     options.success(true);
            var tMatches = request.params.tMatches;
            console.log("player1 after match delete..." + player1);
            console.log(player1);
            var queryUser = new Parse.Query("User");
            queryUser.get(player1.id, {
                success: function(playerOne) {
                    console.log("player1 after geting from user ..." + playerOne);
                    console.log(playerOne);
                    _getMatches(request, options, tMatches, playerOne);
                },
                error: options.error});
        },
        error: function(object, error)
        {
            // The object was not retrieved successfully.
            // error is a Parse.Error with an error code and description.
            console.log(" error in match deleting ...");
            console.log(error);
            options.error(error);
        }
    });
};





