// ===========================================================================================================
// UserHandler.lua
//
// To handle User updateuser, addfriend ,deletefriend ,getUsers by parse cloud code for iOS and Android
//
// Version 1.0
//
// Developed by: Qasim Farooqi (Swenggco Software)
// ===========================================================================================================

var PushHandler = require('cloud/PushHandler.js');
var ParseCode = require('cloud/ParseCode.js');

_log = function(message, player) {
    console.log(player.get("username") + "$ " + message);
};

_updateUser = function(keyValues, user, options) {
    for (var k in keyValues) {
        if (keyValues.hasOwnProperty(k)) {
            user.set(k, keyValues[k]);
        }
    }
    user.save(null, {
        success: function(updatedUser) {
            // Return the game
            console.log("_updateUser successfully");
            options.success(true);
        },
        error: function(error) {
            options.error(error + "error in _updateUser");
        }
    });
};

exports.updateUser = function(keyValues, user, options) {
    _updateUser(keyValues, user, options);
};

_addFriend = function(player1, player2, options) {

    Parse.Cloud.useMasterKey();
    var player1FrdList = player1.get("friends");
    var player2FrdList = player2.get("friends");
    var index1 = 0;
    var index2 = 0;
//    console.log("player1FrdList.length+1"+player1FrdList.count());
    console.log(player1FrdList);
//    console.log("player2FrdList.length+1"+player2FrdList.length+1);

    if (!player1FrdList) {
        player1FrdList = [];
        index1 = 0;
    } else {
        index1 = player1FrdList.length;
    }

    if (!player2FrdList) {
        player2FrdList = [];
        index2 = 0;
    } else {
        index2 = player2FrdList.length;
    }

    player1FrdList[index1] = player2.id;
    player2FrdList[index2] = player1.id;
    player1.set("friends", player1FrdList);
    player1.save(null, {
        success: function(playerOne) {
            _log("playerOne frdlist updated", playerOne);
            player2.set("friends", player2FrdList);
            player2.save(null, {
                success: function(playerTwo) {
                    _log("playerTwo frdlist updated", playerTwo);
                    var valuesForPush = {"condition": "friend", "code": ParseCode.friend, "friendId": player2.id};
                    PushHandler._sendPush(player2, player1.get("name") + " added you as a friend. Get ready of new challenges.", valuesForPush);
                    options.success(player1);
                },
                error: options.error});
        },
        error: options.error});
};


exports.addFriend = function(request, options)
{
    var player = request.user;
    var friendId = request.params.friendId;
    var query = new Parse.Query("User");
    query.get(friendId, {
        success: function(friend) {
            _addFriend(player, friend, options);
        },
        error: options.error});
};


exports.signOutUser = function(request, options) {
    var user = request.user;
    var installationId = request.params.installationId;
    var deviceToken = request.params.deviceToken;
    var query = new Parse.Query("User");
    query.get(user.id, {
        success: function(myUser) {
            if (installationId == myUser.get("installationId") || myUser.get("deviceToken") || myUser.get("userDevice")) {
                //update to null 
                var keyValues = [];
                keyValues["installationId"] = null;
                keyValues["userDevice"] = null;
                keyValues["deviceToken"] = null;
                _updateUser(keyValues, myUser, options);
            } else {
                options.success(true);
            }

        },
        error: options.error});
};



exports.searchUser = function(request, options)
{
    var player = request.user;
    var searchStr = request.params.searchQuery;
    var query1 = new Parse.Query("User");
    query1.notEqualTo("objectId", player.id);
    query1.notEqualTo("isAgent", true);
    query1.notContainedIn("friends", [player.id]);
    query1.limit(1000);
    query1.find().then(function(results) {
        var searchedUser = [];
        if (results.length > 0) {
            for (var j = 0; j < results.length; j++) {

                var username = results[j].get("username").toLowerCase();
                var name = results[j].get("name").toLowerCase();
                var email = results[j].get("email");
                var searchStrLower = searchStr.toLowerCase();

                if (name.match(searchStrLower)) {
                    searchedUser[searchedUser.length] = results[j];
                } else if (username.match(searchStrLower)) {
                    searchedUser[searchedUser.length] = results[j];
                } else if (email && email.match(searchStrLower)) {
                    searchedUser[searchedUser.length] = results[j];
                }
            }
        }
        options.success(searchedUser);
    }, function(error) {
        options.error(error);
    });
};




exports.getMyFriends = function(request, options)
{
    var player = request.user;
    var query = new Parse.Query("User");
    query.notEqualTo("objectId", player.id);
    query.limit(1000);
    query.ascending("name");
    query.containedIn("friends", [player.id]);
    query.find().then(function(results) {
        options.success(results);
    }, function(error) {
        options.error(error);
    });
};





_deleteFriend = function(player1, player2, options) {

    Parse.Cloud.useMasterKey();
    var player1FrdList = player1.get("friends");
    var player2FrdList = player2.get("friends");
    console.log(player1FrdList);
    if (!player1FrdList || player1FrdList.length == 0) {
        options.success("user have not any friend");
    } else {

        if (!player2FrdList || player2FrdList.length == 0) {
            console.log(player2FrdList + " player2FrdList hava not any frd");
            options.success("other user have not any friend");
        } else {


            for (var i = 0; i < player1FrdList.length; i++) {

                if (player1FrdList[i] == player2.id) {
                    console.log("player1FrdList[i]" + player1FrdList[i] + "find");
                    player1FrdList.splice(i, 1);
                    break;
                }
            }

            for (var i = 0; i < player2FrdList.length; i++) {
                if (player2FrdList[i] == player1.id) {
                    player2FrdList.splice(i, 1);
                    break;
                }
            }

            player1.set("friends", player1FrdList);
            player1.save(null, {
                success: function(playerOne) {
                    _log("playerOne frdlist updated after remove", playerOne);
                    player2.set("friends", player2FrdList);
                    player2.save(null, {
                        success: function(playerTwo) {
                            _log("playerTwo frdlist updated after remove", playerTwo);
                            options.success(player1);
                        },
                        error: options.error});
                },
                error: options.error});
        }

    }


};




exports.deleteFriend = function(request, options)
{
    var player = request.user;
    var friendId = request.params.friendId;
    var query = new Parse.Query("User");
    query.get(friendId, {
        success: function(friend) {
            _deleteFriend(player, friend, options);
        },
        error: options.error}
    );
};